﻿
// Global variable to hold reference to Web Rule
// (optional, just for the sake of convenience)
var webrule;

// Init everything on page load
$(function ()
{
	// Use Web Rule's global shortcut $ce to get reference to
	// the control by its server ID (not its client ID)
	webrule = $ce("ruleControl");

	// Set action delegates that Web Rule calls on save, load or delete the rule.
	// The functions themselves are declared later in this script
	webrule.setClientActions(loadRule, deleteRule, saveRule);

	// Use the evaluateRule method as Test button's click event handler
	$("#btnTest").click(evaluateRule);

	// Attach change event handler to the "themes" drop down list
	$("#ddlTheme").change(loadSettings);

	// Attach click even handlers to the "help" and "dots" check boxes
	$("#chkHelp").click(loadSettings);
	$("#chkDots").click(loadSettings);

	// Finally, call the GetSettings() web method to get control's settings
	loadSettings();
});

// Method that requests Web Rule UI settings from the server
// Call it every time you need to change any of Web Rule's UI settings
function loadSettings()
{
	// Make sure to clear the Rule Area
	webrule.clear();
	// On this page, we only have 3 Web Rule settings: theme, help and line dots.
	// Get their values and pass it all to the server.
	var theme = $("#ddlTheme option:selected").val();
	var help = $("#chkHelp").is(":checked"), dots = $("#chkDots").is(":checked");
	// Call the LoadSettings web method (defined in /Ajax.aspx.cs) to get the settings.
	// Note that the data we are passing must be serialized (JSON.stringify)
	post("/LoadSettings", JSON.stringify({ theme: theme, showHelpString: help, showLineDots: dots }), settingsLoaded);
};

// Method that handles server response to the loadSettings call
function settingsLoaded(settings)
{
	// Pass the received json string to Web Rule
	webrule.loadSettings(settings);
};

// Action method that loads rule from the storage.
// It has to have one parameter
function loadRule(ruleId)
{
	// Call the LoadRule web method defined in /Ajax.aspx.cs
	post("/LoadRule", JSON.stringify({ ruleId: ruleId }), ruleLoaded);
};

// Handler of the successful loadRule() call
function ruleLoaded(ruleData)
{
	// Load the rule into the Rule Area by passing Web Rule the json
	// string returned by the server.
	webrule.loadRule(ruleData);
	// Notify the user that the rule was loaded
	$("#lblInfo").text("Rule is loaded");
};

// Action method that deletes the current rule from the storage.
// It has to define one parameter
function deleteRule(ruleId)
{
	// Call the DeleteRule web method defined in /Ajax.aspx.cs
	post(
		"/DeleteRule",
		JSON.stringify({ ruleId: ruleId }),
		function (e)
		{
			// Let the Web Rule know that there were no errors and
			// the rule was deleted successfully
			webrule.deleted(ruleId);
			// Clear the Rule Area (we just deleted the rule,
			// there is no point of displaying it anymore)
			webrule.clear();
			// Notify the user that the rule was deleted
			$("#lblInfo").text("Rule was deleted successfully");
		});
};

// Action method that saves the new or existing rule in the storage.
// It has to define one parameter
function saveRule(ruleData)
{
	// Call the SaveRule web method defined in /Ajax.aspx.cs
	post("/SaveRule", JSON.stringify({ ruleData: ruleData }), ruleSaved);
};

// Handler of the successful saveRule() call
function ruleSaved(data)
{
	// Remember that MS implementation of web methods that return complex types
	// serialize the returned object in a complex client object with property "d"
	// that holds the actual data returned by the server. Check for that when receiving
	// the saveRule() response.
	data = data.d || data;
	if (data.IsRuleEmpty) $("#lblInfo").text("The rule is empty"); // The Rule Area is empty
	else if (!data.IsRuleValid)
	{
		// The rule is invalid. Pass the received invalid client data to Web Rule
		webrule.loadInvalids(data.ClientInvalidData);
		//If the Help String is enabled, Web Rule does notify the user that the rule is invalid.
		// But here we also use the info label to do the same. Just for the sake of consistency :)
		$("#lblInfo").text("The rule is not valid");
	}
	else
	{
		// Server returns rule ID using the Output property of ProcessingResult C# type.
		// Web Rule needs this ID if the saved rule was a new rule.
		webrule.saved(data.Output);
		// Inform the user
		$("#lblInfo").text("The rule was saved successfully");
	}
};

// Function that handles the evaluation of the current rule. It's attached to the Evaluate button
// as its "click" handler (see the init function on top of the script). Normally you wouldn't want
// to evaluate rules in a web application but we do this here simply to demonstrate how it can be done.
function evaluateRule()
{
	// The webrule.extract() method returns rule's data; getSource() method is declared later in
	// this script; it returns values of the Patient's form controls.
	var data = JSON.stringify({ patient: getSource(), ruleData: webrule.extract() });
	// Call the EvaluateRule web method defined in /Ajax.aspx.cs
	post("/EvaluateRule", data, ruleEvaluated);
};

// Handler of the successful evaluateRule() call
function ruleEvaluated(data)
{
	data = data.d || data;
	if (data.IsRuleEmpty) $("#lblInfo").text("The rule is empty"); // The Rule Area was empty
	else if (!data.IsRuleValid)
	{
		// The rule is not valid, pass invalid data to Web Rule
		webrule.loadInvalids(data.ClientInvalidData);
		// Notify the user
		$("#lblInfo").text("The rule is not valid");
	}
	else $("#lblInfo").text(data.Output); // Display the evaluation output returned by the server
};

// Utility wrapper that handles HTTP posts to the server.
// jQuery is used here as an example; you can use any framework
// that wraps calls to xmlHttpRequest
function post(url, data, delegate)
{
	$.ajax({
		type: "POST",
		url: window.location.pathname + url,
		contentType: "application/json; charset=utf-8",
		dataType: "json",
		data: data,
		cache: false,
		success: delegate,
		error: error /*all errored calls will be delegated to the error() method declared below*/
	});
};

// Handler of errored server calls
function error(e)
{
	if (!e) throw Eror("Generic server error.");
	else
	{
		// We are using properties of the complex "e" error object to notify
		// the user about the error. This error handling has nothing
		// to do with Web Rule and exists here only for the sake of completeness.
		if (e.responseText) $("#lblInfo").text($.parseJSON(e.responseText).Message);
		else throw Error(e);
	}
};

// Function that returns the client representation of the C# Patient object,
// filled with data from the Patient's form. Property names of this client object
// must match those in the Patient C# source object. See also the EvaluateRule
// web method defined in /Ajax.aspx.cs file
function getSource()
{
	function Patient() { };
	function Address() { };

	var p = new Patient();
	p.FirstName = getText("txtFirstName");
	p.LastName = getText("txtLastName");
	p.Email = getText("txtEmail");
	var d = getText("txtDOB");
	p.Gender = $("#ddlGender option:selected").val();
	if (d != null)
	{
		try { p.DOB = new Date(d); }
		catch (e) { p.DOB = null; };
	}
	p.EducationTypeID = $("#ddlEducation option:selected").val();

	p.Home = new Address();
	p.Home.Street = getText("txtHomeStreet");
	p.Home.City = getText("txtHomeCity");
	p.Home.Zip = getText("txtHomeZip");
	p.Home.State = $("#ddlHomeState option:selected").val();

	p.Work = new Address();
	p.Work.Street = getText("txtWorkStreet");
	p.Work.City = getText("txtWorkCity");
	p.Work.Zip = getText("txtWorkZip");
	p.Work.State = $("#ddlWorkState option:selected").val();

	p.Pulse = getText("txtPulse");
	p.SystolicPressure = getText("txtSysPressure");
	p.DiastolicPressure = getText("txtDiaPressure");
	p.Temperature = getText("txtTemperature");

	p.Headaches = $("#chkHeadache").is(":checked");
	p.Allergies = $("#chkAllergy").is(":checked");
	p.Tobacco = $("#chkSmoker").is(":checked");
	p.Alcohol = $("#chkAlcohol").is(":checked");

	return p;
};

// Utility method that makes sure that empty
// textbox values will be returned as nulls
function getText(id)
{
	var v = $("#" + id).val();
	return v.length == 0 ? null : v;
};